#ifndef OBJECT_INCLUDED // -*- C++ -*-
#define OBJECT_INCLUDED

/*--------------------------------------------------------------------------

   object.h v3.0 21/March/2008
   Copyright (C) 2008 J. Ribelles

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

  --------------------------------------------------------------------------*/

#include <list>

class Transformation {

 private:
  int   type;            // type: scale, translate, rotate
  float x, y, z, alpha;  // parameters of the transformation

 public:
  Transformation (void) {}
  Transformation (int _type, float _x, float _y, float _z, float _alpha) : type(_type), x(_x), y(_y), z(_z), alpha(_alpha) {}
  virtual ~Transformation (void) {}

  void apply (void);
  void print (FILE *fc);
  void printSave (FILE *fc);

  void get (int &_type, float &_x, float &_y, float &_z, float &_alpha) { 
    _type= type; _x= x; _y= y; _z= z; _alpha= alpha;
  };
  int  getType (void) { return type;}
};


class Primitive {

 private:
  int   id;              // id of the primitive
  int   type;            // type: sphere, cube, etc...
  float x1, x2, x3, x4;  // parameters of the primitive: radio, etc...
  list<Transformation> transformations; // list of transformations

 public:
  Primitive (void) : transformations() {}
  Primitive (int _id, int _type, float _x1, float _x2, float _x3, float _x4) : id(_id), type(_type), x1(_x1), x2(_x2), x3(_x3), x4(_x4), transformations(0) {}
  virtual ~Primitive (void) {}

  int  getId (void)   {return id;}
  int  getType (void) {return type;}
  void draw (void);
  void print (FILE *fc, int indent);
  void printSave (FILE *fc);

  void getFirstTransformation (int &_type, float &_x, float &_y, float &_z, float &_alpha) {
    if (!transformations.empty()) transformations.begin()->get (_type, _x, _y, _z, _alpha);
  }
  void addTransformation (int type, float x, float y, float z, float alpha) {
    Transformation theTransformation (type, x, y, z, alpha);
    transformations.push_front (theTransformation);
  }
  void eraseTransformation (void)  { if (!transformations.empty()) transformations.pop_front(); }
  int  emptyTransformations (void) { return (transformations.empty()); }
  void applyTransformations (void) {
    if (emptyTransformations()) return;
    for (list<Transformation>::iterator itera= transformations.begin(); itera!= transformations.end(); itera++)
      itera->apply();
  }
  void printTransformations (FILE *fc) {
    if (emptyTransformations()) return;
    for (list<Transformation>::iterator itera= transformations.begin(); itera!= transformations.end(); itera++)
      itera->print(fc);
  }
  void printTransformationsSave (FILE *fc){
    list<Transformation>::iterator itera;
    if (emptyTransformations()) return;
    for (itera= --transformations.end(); itera!= transformations.begin(); itera--)
      itera->printSave(fc);
    itera->printSave(fc);
  }  
};


class Object {

 private:

  list<Primitive> primitives;
  list<Primitive>::iterator current;
  int cont;

 public:

  Object (void) : primitives(0), cont(0) {current= primitives.begin();}
  virtual ~Object (void) {}

  int  add        (int type, float x1, float x2, float x3, float x4);
  void draw       (void);
  void drawSelect (void);
  void erase      (void);
  void setCurrent (int id);
  void save       (void);
  bool load       (const char *filename);

  void applyTransformations (void) { if (!primitives.empty()) current->applyTransformations(); }
  void addTransformation (int type, float x, float y, float z, float alpha) { if (!primitives.empty()) current->addTransformation (type, x, y, z, alpha); }
  void getFirstTransformation (int &_type, float &_x, float &_y, float &_z, float &_alpha) { if (!primitives.empty()) current->getFirstTransformation (_type, _x, _y, _z, _alpha); }
  void eraseTransformation (void)  { if (!primitives.empty()) current->eraseTransformation(); }
  bool empty (void)                { return (primitives.empty()); }
  bool emptyTransformations (void) { return (current->emptyTransformations()); }
};

#endif
